﻿using System;
using System.Text;
using System.Collections.Generic;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using ServiceTrackerPresenter;
using ServiceTrackerLogic;

namespace ServiceTrackerPresenterTests
{
	/// <summary>
	/// Summary description for UnitTest1
	/// </summary>
	[TestClass]
	public class AccountPresenterTests
	{
		[TestCleanup]
		public void TearDown()
		{
			//Clean up AccountStore
			Accounts.AccountStore = null;
		}

		[TestMethod]
		public void PresenterRegistersAsListenerForSearchCustomersEvent()
		{
			IAccountView view = new StubAccountView();
			Assert.AreEqual(0, ((StubAccountView)view).SearchCustomersRequestedListenerCount);
			AccountPresenter presenter = new AccountPresenter(view);
			int expectedListenersCount = 1;
			int actualListenersCount =
				((StubAccountView)view).SearchCustomersRequestedListenerCount;
			Assert.AreEqual(expectedListenersCount, actualListenersCount);
		}

		[TestMethod]
		public void PresenterRegistersAsListenerForLoadEvent()
		{
			IAccountView view = new StubAccountView();
			Assert.AreEqual(0, ((StubAccountView)view).AccountViewLoadListenerCount);
			AccountPresenter presenter = new AccountPresenter(view);
			int expectedListenersCount = 1;
			int actualListenersCount =
				((StubAccountView)view).AccountViewLoadListenerCount;
			Assert.AreEqual(expectedListenersCount, actualListenersCount);
		}

		[TestMethod]
		public void AccountGridDataSourceSetDuringViewLoad()
		{
			IAccountView view = new StubAccountView();
			AccountPresenter presenter = new AccountPresenter(view);
			Assert.IsNull(view.AccountGrid.DataSource);
			((StubAccountView)view).FireAccountViewLoadEvent();
			Assert.IsNotNull(view.AccountGrid.DataSource);
		}

		[TestMethod]
		public void GridDSSetToUpdatedAccountsWhenSearchEventFired()
		{
			IAccountView view = new StubAccountView();
			AccountPresenter presenter = new AccountPresenter(view);
			((StubAccountView)view).FireAccountViewLoadEvent();
			view.SearchCriteria = "813-555-1234";
			((StubAccountView)view).FireSearchCustomersRequestedEvent();
			Account expectedAccount = Account.CreateAccount("Test User", "813-555-1234");
			Accounts accounts = view.AccountGrid.DataSource as Accounts;
			Assert.AreEqual(1, accounts.Count());
			Assert.AreEqual(expectedAccount, accounts[0]);
		}

		[TestMethod]
		public void PresenterRegistersAsListenerForAddCustomerRequestedEvent()
		{
			IAccountView view = new StubAccountView();
			Assert.AreEqual(0, ((StubAccountView)view).AddAccountRequestedListenerCount);
			AccountPresenter presenter = new AccountPresenter(view);
			int expectedListenersCount = 1;
			int actualListenersCount =
				((StubAccountView)view).AddAccountRequestedListenerCount;
			Assert.AreEqual(expectedListenersCount, actualListenersCount);
		}

		[TestMethod]
		public void PresenterRequestsNewWindowFromViewWhenAddCustomerRequested()
		{
			IAccountView view = new StubAccountView();
			AccountPresenter presenter = new AccountPresenter(view);
			((StubAccountView)view).FireAddAccountRequestedEvent();
			Assert.IsTrue(((StubAccountView)view).NewAccountWindowRequested);
		} 

		[TestMethod]
		public void DataGridSourceMatchesAccountsCollectionOnLoad()
		{
			IAccountView view = new StubAccountView();
			AccountPresenter presenter = new AccountPresenter(view);
			((StubAccountView)view).FireAccountViewLoadEvent();
			Assert.AreEqual(Accounts.AccountStore, view.AccountGrid.DataSource);
		}

		[TestMethod]
		public void DataGridUpdatedWhenAccountsCollectionChanged()
		{
			IAccountView view = new StubAccountView();
			AccountPresenter presenter = new AccountPresenter(view);
			((StubAccountView)view).FireAccountViewLoadEvent();
			Accounts.AccountStore.Add(Account.CreateAccount("Faked Out", "813-111-2222"));
			Assert.AreEqual(Accounts.AccountStore, view.AccountGrid.DataSource);
		}
	}

	class StubAccountView : IAccountView
	{
		public event AccountViewEventDelegate SearchCustomersRequested;
		public event AccountViewEventDelegate AccountViewLoad;
		public event AccountViewEventDelegate AddAccountRequested;
		private IAccountGrid accountGrid;

		public bool NewAccountWindowRequested = false;

		public StubAccountView()
		{
			accountGrid = new StubAccountGrid();
		}

		public IAddAccountView AddAccountForm
		{
			get { return null; }
		}
			
		public void OpenAccountWindow()
		{
			NewAccountWindowRequested = true;
		}

		public void FireAddAccountRequestedEvent()
		{
			if (AddAccountRequested != null)
				AddAccountRequested(this, this);
		}

		public IAccountGrid AccountGrid
		{
			get { return accountGrid; }
		}

		public void FireAccountViewLoadEvent()
		{
			if (AccountViewLoad != null)
				AccountViewLoad(this, this);
		}

		public void FireSearchCustomersRequestedEvent()
		{
			if (SearchCustomersRequested != null)
				SearchCustomersRequested(this, this);
		}

		private string searchCriteria;

		public string SearchCriteria
		{
			get { return searchCriteria; }
			set { searchCriteria = value; }
		}


		public int AddAccountRequestedListenerCount
		{
			get
			{
				if (AddAccountRequested != null)
				{
					return AddAccountRequested.GetInvocationList().Count();
				}
				return 0;
			}
		}

		public int SearchCustomersRequestedListenerCount
		{
			get
			{
				if (SearchCustomersRequested != null)
				{
					return SearchCustomersRequested.GetInvocationList().Count();
				}
				return 0;
			}
		}

		public int AccountViewLoadListenerCount
		{
			get
			{
				if (AccountViewLoad != null)
				{
					return AccountViewLoad.GetInvocationList().Count();
				}
				return 0;
			}
		}

	}

	class StubAccountGrid : IAccountGrid
	{
		private object dataSource;

		public object DataSource
		{
			get { return dataSource; }
			set { dataSource = value; }
		}
	}
}
